@@ -31,6 +31,7 @@ import time
 import uuid
 import yaml
 
+from ironicclient import client as ir_client
 from keystoneauth1 import session
 from keystoneauth1 import exceptions as ks_exceptions
 from keystoneclient import discover
@@ -78,6 +79,7 @@ class Paths(object):
 PATHS = Paths()
 DEFAULT_LOG_LEVEL = logging.DEBUG
 DEFAULT_LOG_FORMAT = '%(asctime)s %(levelname)s: %(message)s'
+DEFAULT_NODE_RESOURCE_CLASS = 'baremetal'
 LOG = None
 CONF = cfg.CONF
 COMPLETION_MESSAGE = """
@@ -156,12 +158,10 @@ _opts = [
                 default=[],
                 help=('List of ntp servers to use.')),
     cfg.StrOpt('overcloud_domain_name',
-               default='',
+               default='localdomain',
                help=('DNS domain name to use when deploying the overcloud. '
                      'The overcloud parameter "CloudDomain" must be set to a '
-                     'matching value. When unset, the installation will try '
-                     'to use the undercloud\'s domain name, and if that fails '
-                     'it\'ll use the default from neutron.conf.')
+                     'matching value.')
                ),
     cfg.StrOpt('undercloud_service_certificate',
                default='',
@@ -552,6 +552,8 @@ def _load_config():
         conf_params += ['--config-file', PATHS.PASSWORD_PATH]
     if os.path.isfile(PATHS.CONF_PATH):
         conf_params += ['--config-file', PATHS.CONF_PATH]
+    else:
+        LOG.warning('%s does not exist. Using defaults.' % PATHS.CONF_PATH)
     CONF(conf_params)
 
 
@@ -1368,18 +1370,63 @@ def _delete_default_flavors(nova):
             nova.flavors.delete(f.id)
 
 
-def _ensure_flavor(nova, name, profile=None):
-    try:
+def _ensure_flavor(nova, existing, name, profile=None):
+    rc_key_name = 'resources:CUSTOM_%s' % DEFAULT_NODE_RESOURCE_CLASS.upper()
+    keys = {
+        # First, make it request the default resource class
+        rc_key_name: "1",
+        # Then disable scheduling based on everything else
+        "resources:DISK_GB": "0",
+        "resources:MEMORY_MB": "0",
+        "resources:VCPU": "0"
+    }
+
+    if existing is None:
         flavor = nova.flavors.create(name, 4096, 1, 40)
-    except exceptions.Conflict:
+
+        keys['capabilities:boot_option'] = 'local'
+        if profile is not None:
+            keys['capabilities:profile'] = profile
+        flavor.set_keys(keys)
+        message = 'Created flavor "%s" with profile "%s"'
+
+        LOG.info(message, name, profile)
+    else:
         LOG.info('Not creating flavor "%s" because it already exists.', name)
-        return
-    keys = {'capabilities:boot_option': 'local'}
-    if profile is not None:
-        keys['capabilities:profile'] = profile
-    flavor.set_keys(keys)
-    message = 'Created flavor "%s" with profile "%s"'
-    LOG.info(message, name, profile)
+
+        # NOTE(dtantsur): it is critical to ensure that the flavors request
+        # the correct resource class, otherwise scheduling will fail.
+        old_keys = existing.get_keys()
+        for key in old_keys:
+            if key.startswith('resources:CUSTOM_') and key != rc_key_name:
+                LOG.warning('Not updating flavor %s, as it already has a '
+                            'custom resource class %s. Make sure you have '
+                            'enough nodes with this resource class.',
+                            existing.name, key)
+                return
+
+        # Keep existing values
+        keys.update(old_keys)
+        existing.set_keys(keys)
+        LOG.info('Flavor %s updated to use custom resource class %s',
+                 name, DEFAULT_NODE_RESOURCE_CLASS)
+
+
+def _ensure_node_resource_classes(ironic):
+    for node in ironic.node.list(limit=-1, fields=['uuid', 'resource_class']):
+        if node.resource_class:
+            if node.resource_class != DEFAULT_NODE_RESOURCE_CLASS:
+                LOG.warning('Node %s is using a resource class %s instead '
+                            'of the default %s. Make sure you use the correct '
+                            'flavor for it.', node.uuid, node.resource_class,
+                            DEFAULT_NODE_RESOURCE_CLASS)
+            continue
+
+        ironic.node.update(node.uuid,
+                           [{'path': '/resource_class', 'op': 'add',
+                             'value': DEFAULT_NODE_RESOURCE_CLASS}])
+        LOG.info('Node %s resource class was set to %s',
+                 node.uuid, DEFAULT_NODE_RESOURCE_CLASS)
 
 
 def _copy_stackrc():
@@ -1441,6 +1488,72 @@ def _migrate_plans(mistral, swift, plans):
             mistral.environments.delete(plan)
 
 
+def _wait_for_mistral_execution(timeout_at, mistral, execution, message='',
+                                fail_on_error=False):
+    while time.time() < timeout_at:
+        exe = mistral.executions.get(execution.id)
+        if exe.state == "RUNNING":
+            time.sleep(5)
+            continue
+        if exe.state == "SUCCESS":
+            return
+        else:
+            exe_out = ""
+            exe_created_at = time.strptime(exe.created_at,
+                                           "%Y-%m-%d %H:%M:%S")
+            ae_list = mistral.action_executions.list()
+            for ae in ae_list:
+                if ((ae.task_name == "run_validation") and
+                    (ae.state == "ERROR") and
+                    (time.strptime(ae.created_at,  "%Y-%m-%d %H:%M:%S") >
+                     exe_created_at)):
+                    task = mistral.tasks.get(ae.task_execution_id)
+                    task_res = task.to_dict().get('result')
+                    exe_out = "%s %s" % (exe_out, task_res)
+            error_message = "ERROR %s %s Mistral execution ID: %s" % (
+                message, exe_out, execution.id)
+            LOG.error(error_message)
+            if fail_on_error:
+                raise RuntimeError(error_message)
+            return
+    else:
+        exe = mistral.executions.get(execution.id)
+        error_message = ("TIMEOUT waiting for execution %s to finish. "
+                         "State: %s" % (exe.id, exe.state))
+        LOG.error(error_message)
+        if fail_on_error:
+            raise RuntimeError(error_message)
+
+
+def _get_session():
+    user, password, project, auth_url = _get_auth_values()
+    auth_kwargs = {
+        'auth_url': auth_url,
+        'username': user,
+        'password': password,
+        'project_name': project,
+        'project_domain_name': 'Default',
+        'user_domain_name': 'Default',
+    }
+    auth_plugin = ks_auth.Password(**auth_kwargs)
+    return session.Session(auth=auth_plugin)
+
+
+def _run_validation_groups(groups=[], mistral_url='', timeout=360,
+                           fail_on_error=False):
+    sess = _get_session()
+    mistral = mistralclient.client(mistral_url=mistral_url, session=sess)
+    LOG.info('Starting and waiting for validation groups %s ', groups)
+    execution = mistral.executions.create(
+        'tripleo.validations.v1.run_groups',
+        workflow_input={'group_names': groups}
+    )
+    fail_message = ("error running the validation groups %s " % groups)
+    timeout_at = time.time() + timeout
+    _wait_for_mistral_execution(timeout_at, mistral, execution, fail_message,
+                                fail_on_error)
+
+
 def _create_default_plan(mistral, plans, timeout=360):
     plan_name = 'overcloud'
     queue_name = str(uuid.uuid4())
@@ -1454,29 +1567,13 @@ def _create_default_plan(mistral, plans, timeout=360):
         'tripleo.plan_management.v1.create_default_deployment_plan',
         workflow_input={'container': plan_name, 'queue_name': queue_name}
     )
-
     timeout_at = time.time() + timeout
-
-    while time.time() < timeout_at:
-        exe = mistral.executions.get(execution.id)
-        if exe.state == "RUNNING":
-            time.sleep(5)
-            continue
-        if exe.state == "SUCCESS":
-            return
-        else:
-            raise RuntimeError(
-                "Failed to create the default Deployment Plan. Please check "
-                "the create_default_deployment_plan execution in Mistral with "
-                "`openstack workflow execution list`.")
-    else:
-        exe = mistral.executions.get(execution.id)
-        LOG.error("Timed out waiting for execution %s to finish. State: %s",
-                  exe.id, exe.state)
-        raise RuntimeError(
-            "Timed out creating the default Deployment Plan. Please check "
-            "the create_default_deployment_plan execution in Mistral with "
-            "`openstack workflow execution list`.")
+    fail_message = ("error creating the default Deployment Plan %s "
+                    "Check the create_default_deployment_plan execution "
+                    "in Mistral with openstack workflow execution list " %
+                    plan_name)
+    _wait_for_mistral_execution(timeout_at, mistral, execution, fail_message,
+                                fail_on_error=True)
 
 
 def _prepare_ssh_environment(mistral):
@@ -1509,16 +1606,7 @@ def _post_config_mistral(instack_env, mistral, swift):
 def _post_config(instack_env):
     _copy_stackrc()
     user, password, project, auth_url = _get_auth_values()
-    auth_kwargs = {
-        'auth_url': auth_url,
-        'username': user,
-        'password': password,
-        'project_name': project,
-        'project_domain_name': 'Default',
-        'user_domain_name': 'Default',
-    }
-    auth_plugin = ks_auth.Password(**auth_kwargs)
-    sess = session.Session(auth=auth_plugin)
+    sess = _get_session()
     # TODO(andreykurilin): remove this check with support of novaclient 6.0.0
     if nc.__version__[0] == "6":
         nova = novaclient.Client(2, user, password, project, auth_url=auth_url)
@@ -1526,15 +1614,22 @@ def _post_config(instack_env):
         nova = novaclient.Client(2, user, password, auth_url=auth_url,
                                  project_name=project)
 
+    ironic = ir_client.get_client(1, session=sess,
+                                  os_ironic_api_version='1.21')
+
     _configure_ssh_keys(nova)
     _delete_default_flavors(nova)
 
-    _ensure_flavor(nova, 'baremetal')
-    _ensure_flavor(nova, 'control', 'control')
-    _ensure_flavor(nova, 'compute', 'compute')
-    _ensure_flavor(nova, 'ceph-storage', 'ceph-storage')
-    _ensure_flavor(nova, 'block-storage', 'block-storage')
-    _ensure_flavor(nova, 'swift-storage', 'swift-storage')
+    _ensure_node_resource_classes(ironic)
+
+    all_flavors = {f.name: f for f in nova.flavors.list()}
+    for name, profile in [('baremetal', None),
+                          ('control', 'control'),
+                          ('compute', 'compute'),
+                          ('ceph-storage', 'ceph-storage'),
+                          ('block-storage', 'block-storage'),
+                          ('swift-storage', 'swift-storage')]:
+        _ensure_flavor(nova, all_flavors.get(name), name, profile)
 
     mistral_url = instack_env['UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC']
     mistral = mistralclient.client(
@@ -1629,6 +1724,9 @@ def install(instack_root, upgrade=False):
         _run_orc(instack_env)
         _post_config(instack_env)
         _run_command(['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
+        if upgrade and CONF.enable_validations:  # Run post-upgrade validations
+            mistral_url = instack_env['UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC']
+            _run_validation_groups(["post-upgrade"], mistral_url)
     except Exception as e:
         LOG.debug("An exception occurred", exc_info=True)
         LOG.error(FAILURE_MESSAGE,
